==> maze_rl-0.1.1.dev5228-py3-none-any.whl: maze/rllib/maze_rllib_callback_logging.py <==
""" Contains callbacks that enable maze logging capabilities. """
from collections import defaultdict

from ray.rllib.agents import Trainer
from ray.rllib.agents.callbacks import DefaultCallbacks

from maze.core.log_stats.log_stats import increment_log_step, LogStatsLevel, LogStatsAggregator, \
    register_log_stats_writer, get_stats_logger
from maze.core.log_stats.log_stats_writer_console import LogStatsWriterConsole
from maze.core.log_stats.log_stats_writer_tensorboard import LogStatsWriterTensorboard


class MazeRLlibLoggingCallbacks(DefaultCallbacks):
    """Callbacks to enable Maze-style logging."""

    def __init__(self):
        super().__init__()
        self.epoch_stats = None

    def init_logging(self) -> None:
        """Initialize logging.

        This needs to be done here as the on_train_result is not called in the main process, but in a worker process.

        Relies on the following:
          - There should be only one Callbacks object per worker process
          - The on train result should always be called in the same worker (if this is not the case,
            this system should still handle it, but it might mess things up)
        """
        assert self.epoch_stats is None, "Init logging should be called only once"

        # Local epoch stats -- stats from all envs will be collected here together
        self.epoch_stats = LogStatsAggregator(LogStatsLevel.EPOCH)
        self.epoch_stats.register_consumer(get_stats_logger("train"))

        # Initialize Tensorboard and console writers
        register_log_stats_writer(LogStatsWriterTensorboard(log_dir='../../maze_rllib', tensorboard_render_figure=True))
        register_log_stats_writer(LogStatsWriterConsole())

    def on_train_result(self, trainer: Trainer, result: dict, **kwargs) -> None:
        """Aggregates stats of all rollouts in one local aggregator and then writes them out.
        Called at the end of Trainable.train().

        :param trainer: Current model instance.
        :param result: Dict of results returned from model.train() call.
            You can mutate this object to add additional metrics.
        :param kwargs: Forward compatibility placeholder.
        """

        # Initialize the logging for this process if not done yet
        if self.epoch_stats is None:
            print("Initializing logging of train results")
            self.init_logging()

        # The main local aggregator should be empty
        #  - No stats should be collected here until we manually add them
        #  - Stats from the last call should be cleared out already (written out to the logs)
        assert self.epoch_stats.input == {}, "input should be empty at the beginning"

        # Get the epoch stats from the individual rollouts
        epoch_aggregators = trainer.workers.foreach_worker(
            lambda worker: worker.foreach_env(lambda env: env.get_stats(LogStatsLevel.EPOCH))
        )

        # Collect all episode stats from the epoch aggregators of individual rollout envs in the main local aggregator
        for worker_epoch_aggregator in epoch_aggregators:
            for env_epoch_aggregator in worker_epoch_aggregator:
                # Pass stats from the individual env runs into the main epoch aggregator
                for stats_key, stats_value in env_epoch_aggregator.input.items():
                    self.epoch_stats.input[stats_key].extend(stats_value)

        # clear logs at distributed workers
        def reset_episode_stats(env) -> None:
            """Empty inputs of the individual aggregators and make sure they don't have any consumers"""
            epoch_aggregator = env.get_stats(LogStatsLevel.EPOCH)
            epoch_aggregator.input = defaultdict(list)
            epoch_aggregator.consumers = []

        trainer.workers.foreach_worker(lambda worker: worker.foreach_env(lambda env: reset_episode_stats(env)))

        # Increment log step to trigger epoch logging
        increment_log_step()

==> maze_rl-0.1.4-py3-none-any.whl: maze/rllib/maze_rllib_callback_logging.py <==
""" Contains callbacks that enable maze logging capabilities. """
from collections import defaultdict

from ray.rllib.agents import Trainer
from ray.rllib.agents.callbacks import DefaultCallbacks

from maze.core.log_stats.log_stats import increment_log_step, LogStatsLevel, LogStatsAggregator, \
    register_log_stats_writer, get_stats_logger
from maze.core.log_stats.log_stats_writer_console import LogStatsWriterConsole
from maze.core.log_stats.log_stats_writer_tensorboard import LogStatsWriterTensorboard


class MazeRLlibLoggingCallbacks(DefaultCallbacks):
    """Callbacks to enable Maze-style logging."""

    def __init__(self):
        super().__init__()
        self.epoch_stats = None

    def init_logging(self) -> None:
        """Initialize logging.

        This needs to be done here as the on_train_result is not called in the main process, but in a worker process.

        Relies on the following:
          - There should be only one Callbacks object per worker process
          - The on train result should always be called in the same worker (if this is not the case,
            this system should still handle it, but it might mess things up)
        """
        assert self.epoch_stats is None, "Init logging should be called only once"

        # Local epoch stats -- stats from all envs will be collected here together
        self.epoch_stats = LogStatsAggregator(LogStatsLevel.EPOCH)
        self.epoch_stats.register_consumer(get_stats_logger("train"))

        # Initialize Tensorboard and console writers
        register_log_stats_writer(LogStatsWriterTensorboard(log_dir='.', tensorboard_render_figure=True))
        register_log_stats_writer(LogStatsWriterConsole())

    def on_train_result(self, trainer: Trainer, result: dict, **kwargs) -> None:
        """Aggregates stats of all rollouts in one local aggregator and then writes them out.
        Called at the end of Trainable.train().

        :param trainer: Current model instance.
        :param result: Dict of results returned from model.train() call.
            You can mutate this object to add additional metrics.
        :param kwargs: Forward compatibility placeholder.
        """

        # Initialize the logging for this process if not done yet
        if self.epoch_stats is None:
            print("Initializing logging of train results")
            self.init_logging()

        # The main local aggregator should be empty
        #  - No stats should be collected here until we manually add them
        #  - Stats from the last call should be cleared out already (written out to the logs)
        assert self.epoch_stats.input == {}, "input should be empty at the beginning"

        # Get the epoch stats from the individual rollouts
        epoch_aggregators = trainer.workers.foreach_worker(
            lambda worker: worker.foreach_env(lambda env: env.get_stats(LogStatsLevel.EPOCH))
        )

        # Collect all episode stats from the epoch aggregators of individual rollout envs in the main local aggregator
        for worker_epoch_aggregator in epoch_aggregators:
            for env_epoch_aggregator in worker_epoch_aggregator:
                # Pass stats from the individual env runs into the main epoch aggregator
                for stats_key, stats_value in env_epoch_aggregator.input.items():
                    self.epoch_stats.input[stats_key].extend(stats_value)

        # clear logs at distributed workers
        def reset_episode_stats(env) -> None:
            """Empty inputs of the individual aggregators and make sure they don't have any consumers"""
            epoch_aggregator = env.get_stats(LogStatsLevel.EPOCH)
            epoch_aggregator.input = defaultdict(list)
            epoch_aggregator.consumers = []

        trainer.workers.foreach_worker(lambda worker: worker.foreach_env(lambda env: reset_episode_stats(env)))

        # Increment log step to trigger epoch logging
        increment_log_step()
